/****************************************************************************
                          tool_timetools.cpp
						  -------------------
    begin                : Fri Dec 08 2006
    authors              : Xiang Li
	email                : xiangli@ieee.org
****************************************************************************/

/****************************************************************************
    Implementation of tools for time information.
****************************************************************************/

/** standard headers *******************************************************/
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifdef WIN32
	#include <time.h>
#else
	#include <sys/time.h>
	#include <sys/resource.h>
#endif
/***************************************************************************/

/** headers ****************************************************************/
#include "tool_timetools.h"
/***************************************************************************/

#ifdef WIN32
	#pragma warning(disable : 4996)
#endif

/***************************************************************************/
namespace Utilities
{

CTimeTools::CTimeTools()
{
	Reset();
}

void CTimeTools::Reset()
{
	memset( &m_TimingRecord , 0 , sizeof( m_TimingRecord ) );
#if _DEBUG
	m_bStart = false;
#endif

#ifdef WIN32
	QueryPerformanceFrequency( &m_nFrequency );
#else
#endif
}

CTimeTools::~CTimeTools()
{
}

const char * CTimeTools::GetTimeString() const
{
	static char strTimeInfo[128];
	char timebuf1[128] , timebuf2[128];

#ifdef WIN32
	_strdate( timebuf1 );
	_strtime( timebuf2 );
#else
	time_t now;
	struct tm *l_time;
	now = time ((time_t *) NULL); // Get the system time and put it into 'now' as 'calender time'
	time (&now);
	l_time = localtime (&now);
	strftime( timebuf1 , sizeof( timebuf1 ) , "%d-%b-%Y" , l_time);
	strftime( timebuf2 , sizeof( timebuf2 ) , "%H:%M:%S" , l_time);
#endif
	sprintf( strTimeInfo , "%s,%s" , timebuf1 , timebuf2 );

	return( strTimeInfo );
}

void CTimeTools::StartTiming( bool bNoTiming )
{
#if _DEBUG
	if( m_bStart == true )
		printf( "\n Embedded timing!" );
	m_bStart = true;
#endif

	if( bNoTiming == false )
	{
	#ifdef WIN32		
		FILETIME tDummy;
		HANDLE hProcessID = GetCurrentProcess();
		GetProcessTimes( hProcessID , &tDummy , &tDummy , &m_TimingStart.tKernel , &m_TimingStart.tUser );
		QueryPerformanceCounter( ( LARGE_INTEGER * )( &m_TimingStart.tElapsed ) );
	#else
		struct rusage sRUsage;
		getrusage( RUSAGE_SELF , &sRUsage );
		m_TimingStart.tSystem = sRUsage.ru_stime;
		m_TimingStart.tUser = sRUsage.ru_utime;
		struct timezone tDummy;
		gettimeofday( &m_TimingStart.tElapsed , &tDummy );
	#endif
	}
}

const STimeRecord & CTimeTools::StopTiming( bool bNoTiming )
{
#if _DEBUG
	if( m_bStart == false )
		printf( "\n Embedded timing!" );
	m_bStart = false;
#endif

	if( bNoTiming == false )
	{
	#ifdef WIN32
		FILETIME tDummy;
		HANDLE hProcessID = GetCurrentProcess();
		GetProcessTimes( hProcessID , &tDummy , &tDummy , &m_TimingEnd.tKernel , &m_TimingEnd.tUser );
		QueryPerformanceCounter( ( LARGE_INTEGER * )( &m_TimingEnd.tElapsed ) );
		m_TimingRecord.dElapsedTime = xxTimeDuration( ( LARGE_INTEGER * )( &m_TimingStart.tElapsed ) , ( LARGE_INTEGER * )( &m_TimingEnd.tElapsed ) );
		m_TimingRecord.dUserTime = xxTimeDuration( ( ULARGE_INTEGER * )( &m_TimingStart.tUser ) , ( ULARGE_INTEGER * )( &m_TimingEnd.tUser ) ); 
		m_TimingRecord.dKernelTime = xxTimeDuration( ( ULARGE_INTEGER * )( &m_TimingStart.tKernel ) , ( ULARGE_INTEGER * )( &m_TimingEnd.tKernel ) ); 
	#else
		struct rusage sRUsage;
		getrusage( RUSAGE_SELF , &sRUsage );
		m_TimingEnd.tSystem = sRUsage.ru_stime;
		m_TimingEnd.tUser = sRUsage.ru_utime;
		m_TimingRecord.dUserTime = 1000 * ( m_TimingEnd.tUser.tv_sec - m_TimingStart.tUser.tv_sec ) 
			+ 0.001 * ( m_TimingEnd.tUser.tv_usec - m_TimingStart.tUser.tv_usec );
		m_TimingRecord.dKernelTime = 1000 * ( m_TimingEnd.tSystem.tv_sec - m_TimingStart.tSystem.tv_sec ) 
			+ 0.001 * ( m_TimingEnd.tSystem.tv_usec - m_TimingStart.tSystem.tv_usec );
		struct timezone tDummy;
		gettimeofday( &m_TimingEnd.tElapsed , &tDummy );
		m_TimingRecord.dElapsedTime = 1000 * ( m_TimingEnd.tElapsed.tv_sec - m_TimingStart.tElapsed.tv_sec ) 
			+ 0.001 * ( m_TimingEnd.tElapsed.tv_usec - m_TimingStart.tElapsed.tv_usec );
	#endif
		//m_TimingRecord.dUserTime -= CTimeTools::m_dTimingCost;
		m_TimingRecord.dAccuElapsedTime += m_TimingRecord.dElapsedTime;
		m_TimingRecord.dAccuUserTime += m_TimingRecord.dUserTime;
		m_TimingRecord.dAccuKernelTime += m_TimingRecord.dKernelTime;
		m_TimingRecord.uNumTiming++;
	}

	m_TimingRecord.uNumCalled++;

	return( m_TimingRecord );
}

} // namespace Utilities
/***************************************************************************/

/****************************************************************************
    end-of-file
****************************************************************************/
